@@ -16,9 +16,10 @@ async def get_about(request: Request):
         curl http://localhost/about
     '''
     app: 'DetectorNode' = request.app
+
     return {
         'operation_mode': app.operation_mode.value,
         'state': app.status.state,
         'model_info':  app.detector_logic._model_info,  # pylint: disable=protected-access
-        'target_model': app.target_model,  # pylint: disable=protected-access
+        'target_model': app.target_model.version if app.target_model is not None else 'None',
     }
